@@ -4,9 +4,13 @@ from datetime import datetime
 from threading import Thread
 from multiprocessing import Process
 from functools import wraps
-from json import load as json_load
+from json import load as json_load, dump
 from yaml import safe_load as yaml_load
 from math import sqrt
+from requests import get
+from statistics import mean, median
+from hashlib import sha256
+from js2py import run_file
 
 def get_time() -> dict:
     """
@@ -104,29 +108,31 @@ def prime_factors(number: int) -> list:
     factors = []
     if is_prime(number) or number < 3:
         return factors
+        
     for i in range(2, number):
         if number % i == 0 and is_prime(i):
             factors.append(i)
+            
     return factors
     
-def convert_list_items(old_list: list, type: type):
+def convert_list_items(old_list: list, convert_type: type):
     """
     Info:
         Converts each list item to the type specified
     
     Paramaters:
         old_list: list - List to convert
-        type: type - The type to convert to.
+        convert_type: type - The type to convert to.
 
     Usage:
-        convert_list_items(old_list, type)
+        convert_list_items(old_list, convert_type)
 
     Returns:
         list
     """
     new_list = []
     for item in old_list:
-        new_list.append(type(item))
+        new_list.append(convert_type(item))
     return new_list
 
 def remove_list_duplicates(list: list, amount: int = 1) -> list:
@@ -270,6 +276,34 @@ def wait(before_time: int, after_time: int) -> Callable:
         return wrapper_function
     return decorator
 
+def average_time(amount: int) -> Callable:
+    """
+    Info:
+        Runs the function x amount of times and then collects how long it took to run, after that gets the mean of the list of times to run.
+
+    Paramaters:
+        amount: int - The amount of times you want calculated.
+
+    Usage:
+        @average_time(amount=1)
+        def my_function(a: int, b: str) -> None:
+    """
+    def decorator(function: Callable, *args, **kwargs) -> Callable:
+
+        @wraps(function)
+        def wrapper_function(*args, **kwargs) -> None:
+            times = []
+            for i in range(amount):
+                start_time = time()
+                function(*args, **kwargs)
+                end_time = time()
+                times.append(end_time - start_time)
+            print(f"{function.__name__}: {amount} loops\nTotal time: {sum(times)}\nAverage time: {get_mean(times)}\nLowest Time: {min(times)}\nHighest Time: {max(times)}\nAll times: {', '.join(convert_list_items(times, str))}\n")
+
+        return wrapper_function
+    return decorator
+            
+
 def repeat(amount: int = 1, time: int = None) -> Callable:
     """
     Info:
@@ -581,6 +615,7 @@ def load_json(file_name: str):
     Returns:
         dict
     """
+    
     with open(file_name, 'r') as my_file_raw:
         my_file = json_load(my_file_raw)
         return my_file
@@ -599,6 +634,7 @@ def load_yaml(file_name: str):
     Returns:
         dict
     """
+    
     with open(file_name, 'r') as my_file_raw:
         my_file = yaml_load(my_file_raw)
         return my_file
@@ -617,6 +653,7 @@ def square_root(number: int) -> int:
     Returns:
         int
     """
+    
     return sqrt(number)
 
 def true_or_false() -> dict:
@@ -630,6 +667,7 @@ def true_or_false() -> dict:
     Returns:
         dict
     """
+    
     return {
         'true': [
             'true',
@@ -655,3 +693,236 @@ def true_or_false() -> dict:
             'deny',
             'negative'
         ]}
+
+def scrape_website(url: str) -> str:
+    """
+    Info:
+        Returns the html of a website.
+
+    Paramaters:
+        url: str - The url to fetch and return the html of.
+
+    Usage:
+        scrape_website(url)
+
+    Returns:
+        str        
+    """
+    result = get(url)
+    result_txt = result.text
+    return result_txt
+
+def get_mean(numbers: list) -> int:
+    """
+    Info:
+        Gets the mean of a list of ints.
+
+    Paramaters:
+        numbers: list - The list of numbers to get the mean from.
+
+    Usage:
+        get_mean(numbers)
+
+    Returns:
+        int
+    """
+    return mean(numbers)
+
+def get_median(numbers: list) -> int:
+    """
+    Info:
+        Gets the median of a list of ints.
+
+    Paramaters:
+        numbers: list - The list of numbers to get the median from.
+
+    Usage:
+        get_median(numbers)
+
+    Returns:
+        int
+    """
+    return median(numbers)
+
+class Database:
+    """
+    Info: 
+        Cretes a simple database using JSON
+
+    Options:
+        add - Adds a key and value
+        remove -  Removes a key and value
+        fetch - Fetches a key and value
+        reset - Resets the database
+        update - Updates the database
+
+    Usage:
+        my_databse = Database()
+        my_database.add(key, value)
+    """
+    
+    def __init__(self, create_new_database: bool = True) -> None:
+        """
+        Info:
+            Creates the database when called
+
+        Paramaters:
+            create_new_database: bool - Whether or not to rewrite the database.
+        
+        Usage:
+            my_database = Database()
+        
+        Returns:
+            None
+        """
+        self.database = {}
+        
+        if create_new_database:
+            self.reset()
+
+        self.load()
+        
+    def add(self, key: str, value: Union[str, int, list, dict, tuple]) -> None:
+        """
+        Info:
+            Adds a key and value to the database
+
+        Paramaters:
+            key: str- the key of the item
+            value: Union[str, int, list, dict, tuple] - The value of the key
+
+        Usage:
+            my_database.add(key, value)
+
+        Returns:
+            None
+        """
+        
+        self.database[key] = value
+        self.update()
+
+    def remove(self, key: str) -> None:
+        """
+        Info:
+            Removes a key and value from the database
+
+        Paramaters:
+            key: str- the key of the item
+            value: Union[str, int, list, dict, tuple] - The value of the key
+
+        Usage:
+            my_database.remove(key, value)
+
+        Returns:
+            None
+        """
+        
+        del database[key]
+        self.update()
+
+    def fetch(self, key: str) -> Union[str, int, list, dict, tuple, None]:
+        """
+        Info:
+            Fetches the keys item and returns it
+
+        Paramaters:
+            key: str - The key to fetch and return
+
+        Usage:
+            my_database.fetch(key)
+
+        Returns:
+            Union[str, int, list, dict, tuple]
+        """
+        
+        try:
+            return self.database[key]
+        except Exception:
+            return None
+
+    def reset(self) -> None:
+        """
+        Info:
+            Resets the database
+
+        Usage:
+            my_database.reset()
+
+        Returns:
+            None
+        """
+        
+        self.database = {}
+        self.update()
+    
+    def update(self) -> None:
+        """
+        Info:
+            Updates the database
+
+        Usage:
+            my_database.update()
+
+        Returns:
+            None
+        """
+        
+        with open('database.json', 'w') as my_file:
+            dump(self.database, my_file)
+
+    def load(self) -> None:
+        """
+        Info:
+            Loads the database
+
+        Usage:
+            my_database.load()
+
+        Returns:
+            None
+        """
+        
+        with open('database.json', 'r') as my_file:
+            self.database = json_load(my_file)
+
+def hash_item(item: Union[str, bytes]) -> bytes:
+    """
+    Info:
+        Hashes the given string and returns the hash in bytes
+
+    Paramaters:
+        item: Union[str, bytes] - The item to hash
+    
+    Usage:
+        hash_item(item)
+
+    Returns:
+        bytes
+    """
+    
+    return sha256(item.encode()).hexdigest()
+
+
+def call_js_function(file: str, function: str, args: tuple = ()) -> Union[str, int, float, dict, list, tuple, Callable]:
+    """
+    Info:
+        Calls a javascript function in a js file.
+    
+    Paramaters:
+        file: str - The js file to use.
+        function: str - The function to call.
+        args: tuple - Arguments to pass to the function.
+        requirements: list - Requirements that the js file needs to run.
+
+    Usage:
+        call_js_function(file, function, args=(arg1,))
+  
+    Returns:
+       Union[str, int, float, dict, list, tuple, Callable]
+    """
+    
+    eval_result, js_function = run_file(file)
+    result = js_function[function](*args)
+    
+    return result
+
